/********************************************************************************
 Author : Akshay Kumar B U (akshay.kumar@analog.com)
					India Applications Support Team 

 Date : January 2014

 File name : AD5687.c

 Description : This is an executable file containing all definitions of functions used for operating AD5687 in SPI mode. 

 Hardware Platform : ADuCM360 + AD5687	
********************************************************************************/

/* ADuCM360 interface with AD5687
SYNC	-P1.1
SCLK  -P1.2
SDIN  -P1.3
LDAC  -P1.4
RESET -P1.5
SDO   -P1.6
*/


#include "ADuCM360.h"
#include "ADuCM360Driver.h"
#include "AD5687.h"
#include "ClkLib.h"
#include "IntLib.h"
#include "DioLib.h"
#include "WdtLib.h"
#include "DioLib.h"


//Initializing AD5687 to default settings
void InitializeAD5687(void)
{
	ADuCM360GpioOutput  (AD5687_RESET, PIN_LOW); 
	Delay(100);
	ADuCM360GpioOutput  (AD5687_RESET, PIN_HIGH); 
	Delay(10000);
}

//Function for Initializing/Configuring LDAC pin for Hardware use or software use
/**
	@brief void LdacInitialization(unsigned int LdacMode)
	- Writes the values to LDAC Mask Register
	@param LdacMode :{LDAC_RESPOND_DACA_DACB, LDAC_IGNORE_DACA_RESPOND_DACB, LDAC_RESPOND_DACA_IGNORE_DACB, LDAC_IGNORE_DACA_DACB}
    -LDAC_RESPOND_DACA_DACB          -This makes LDAC respond to both DAC A and DAC B         
    -LDAC_IGNORE_DACA_RESPOND_DACB   -This makes LDAC ignore DAC A and respond to DAC B
    -LDAC_RESPOND_DACA_IGNORE_DACB   -This makes LDAC respond to DAC A and ignore DAC B 
    -LDAC_IGNORE_DACA_DACB           -This makes LDAC ignore both DAC A and DAC B 
	@ no return value
*/
void LdacInitialization(unsigned int LdacMode)
	{
		#ifdef HardwareLDAC
		
			ADuCM360GpioOutput  (AD5687_LDAC, PIN_HIGH); 
			Delay(100);
			ADuCM360GpioOutput  (AD5687_LDAC, PIN_LOW); 
			Delay(100);
		
		#else 
		
			LdacMaskRegister(LdacMode);
		
		#endif
	}
	
	
//Function for writing and updating DAC channel (uses Command 1 or 3 to write based on the requirement)
/**
	@brief void WriteData(unsigned int commandmode, unsigned int DACselect, unsigned int Data)
	- Writes the data to the selected DAC with the selected mode of writing
	@param commandmode: {REG_WRITE_TO_INPUTREG,REG_WRITE_UPDATE_DAC_CHANNEL }
		-REG_WRITE_TO_INPUTREG         //Command 1: Writes the data to the input register(LDAC dependent)
		-REG_WRITE_UPDATE_DAC_CHANNEL  //Command 2: Writes and updates the DAC channel(independent of LDAC)
	@param DACselect :{DAC_A,DAC_B,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_ALL selects DAC A and DAC B	
	@param Data : {0x000000 - 0xFFFFFF}
	@ no return value
**/
void WriteData(unsigned int commandmode, unsigned int DACselect, unsigned int Data)
{
	int RegisterData = 0x000000;
	
	  RegisterData = commandmode + DACselect + Data;  //Merging the data recieved into a single register
	  WriteRegister(RegisterData);
	
}
	
//Function for Updating DAC register from Input register.
/**
	@brief void UpdateDacReg(unsigned int DACselect)
	- Updates DAC register with Input Register.
	@param DACselect :{DAC_A,DAC_B,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_ALL selects DAC A and DAC B
 	@ no return value
**/
	void UpdateDacReg(unsigned int DACselect)
	{
		int RegisterData = 0x000000;
		
		RegisterData = REG_UPDATE_DACREG_WITH_INPUTREG + DACselect ;  //Merging the data recieved into a single register
	  WriteRegister(RegisterData);
	}
	

	
//Function for Powerdown/ Power up DAC
/**
	@brief void PowerDownReg(unsigned int DACAmode, unsigned int DACBmode)
	- Writes the values to Power down/Power up register
	@param DACBmode : {PD_DACB_NORMALMODE, PD_DACB_1K_TO_GND, PD_DACB_100K_TO_GND, PD_DACB_THREE_STATE}
    - PD_DACB_NORMALMODE     //puts DAC B in normal mode
    - PD_DACB_1K_TO_GND      //puts DAC B in powerdown : 1k to GND
    - PD_DACB_100K_TO_GND    //puts DAC B in powerdown : 100k to GND
    - PD_DACB_THREE_STATE    //puts DAC B in powerdown : Three state
  @param DACAmode : {PD_DACA_NORMALMODE, PD_DACA_1K_TO_GND, PD_DACA_100K_TO_GND, PD_DACA_THREE_STATE}
    - PD_DACA_NORMALMODE     //puts DAC A in normal mode
    - PD_DACA_1K_TO_GND      //puts DAC A in powerdown : 1k to GND
    - PD_DACA_100K_TO_GND    //puts DAC A in powerdown : 100k to GND
    - PD_DACA_THREE_STATE    //puts DAC A in powerdown : Three state
 	@ no return value
**/
void PowerDownReg(unsigned int DACAmode, unsigned int DACBmode)
{
		int RegisterData = 0x000000;
	
	  RegisterData = REG_POWERDOWN_UP + DACBmode + DACAmode;  //Merging the data recieved into a single register
	  WriteRegister(RegisterData);
	
}

	
	
//Function for LDAC Mask Register
/**
	@brief void LdacMaskRegister(unsigned int LdacMode)
	- Writes the values to the LDAC Mask Register
	@param LdacMode :{LDAC_RESPOND_DACA_DACB, LDAC_IGNORE_DACA_RESPOND_DACB, LDAC_RESPOND_DACA_IGNORE_DACB, LDAC_IGNORE_DACA_DACB}
    -LDAC_RESPOND_DACA_DACB          //This makes LDAC respond to both DAC A and DAC B         
    -LDAC_IGNORE_DACA_RESPOND_DACB   //This makes LDAC ignore DAC A and respond to DAC B
    -LDAC_RESPOND_DACA_IGNORE_DACB   //This makes LDAC respond to DAC A and ignore DAC B 
    -LDAC_IGNORE_DACA_DACB           //This makes LDAC ignore both DAC A and DAC B 
	@ no return value
**/
void LdacMaskRegister(unsigned int LdacMode)
{
	int RegisterData = 0x000000;
	
	RegisterData = REG_HARDWARE_LDAC_MASK + LdacMode; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}


//Function for Software Reset (Power-on reset)
/**
	@brief void SoftwareReset(void)
	- Writes the values to Software Reset register.
	@param no parameters
 	@ no return value
**/
void SoftwareReset(void)
{
	int RegisterData = 0x000000;
	
	  RegisterData = REG_SOFTWARE_RESET ;  //This setting configures the part for Software Reset
	  WriteRegister(RegisterData);
	
}


//Function for Internal Reference setup
/**
	@brief void InternalReferenceSetup(unsigned int IntRefState)
	- Writes the values to Internal Reference setup register.
	@param IntRefState :{INTREF_ON, INTREF_OFF}
		- INTREF_ON    //Turns ON internal reference
    - INTREF_OFF   //Turns OFF internal reference
 	@ no return value
**/

void InternalReferenceSetup(unsigned int IntRefState)
{
	int RegisterData = 0x000000;
	
	  RegisterData = REG_INTERNAL_REFERENCE + IntRefState;    //Merging the data recieved into a single register
	  WriteRegister(RegisterData);
}


//Function for Daisy Chain Enable
/**
	@brief void DaisyChain(unsigned int DCmode)
	- Writes the values to Daisy Chain Enable register.
	@param DCmode :{DAISY_STANDALONE, DAISY_DCEN_MODE}
		- DAISY_STANDALONE   //Standalone mode(default)
    - DAISY_DCEN_MODE    //Enables Daisy Chain mode
 	@ no return value
**/

void DaisyChain(unsigned int DCmode)
{
	int RegisterData = 0x000000;
	
	  RegisterData = REG_INTERNAL_REFERENCE + DCmode;  //Merging the data recieved into a single register
	  WriteRegister(RegisterData);
	
}


//Function for Enabling Readback register
/**
	@brief void ReadBackEnable(unsigned int DACselect)
	- Writes the values to enable Readback function
	@param DACselect :{DAC_A,DAC_B,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_ALL selects DAC A and DAC B
 	@ no return value
**/
void ReadBackEnable(unsigned int DACselect)
{
	int RegisterData = 0x000000;
	
	  RegisterData = REG_READBACK ;  //This setting configures the part for read mode
	  WriteRegister(RegisterData);
	
	  RegisterData = 0x000000;       
	  WriteRegister(RegisterData);   //Follow this write operation with a second write, a NOP condition, 0x000000
	                                 //During this write, the data from the register is clocked out on the SDO line.
	
}




